@@ -41,10 +41,12 @@ QWaylandLayerSurface::QWaylandLayerSurface(QWaylandLayerShellIntegration *shell,
         setLayer(m_interface->layer());
     });
 
-    set_anchor(m_interface->anchors());
+    setAnchor(m_interface->anchors());
     connect(m_interface, &Window::anchorsChanged, this, [this]() {
-        set_anchor(m_interface->anchors());
+        setAnchor(m_interface->anchors());
+        setDesiredSize(m_window->windowContentGeometry().size());
     });
+
     setExclusiveZone(m_interface->exclusionZone());
     connect(m_interface, &Window::exclusionZoneChanged, this, [this]() {
         setExclusiveZone(m_interface->exclusionZone());
@@ -64,17 +66,7 @@ QWaylandLayerSurface::QWaylandLayerSurface(QWaylandLayerShellIntegration *shell,
         setKeyboardInteractivity(m_interface->keyboardInteractivity());
     });
 
-    QSize size = window->surfaceSize();
-    const Window::Anchors anchors = m_interface->anchors();
-    if ((anchors & Window::AnchorLeft) && (anchors & Window::AnchorRight)) {
-        size.setWidth(0);
-    }
-    if ((anchors & Window::AnchorTop) && (anchors & Window::AnchorBottom)) {
-        size.setHeight(0);
-    }
-    if (size.isValid() && size != QSize(0, 0)) {
-        set_size(size.width(), size.height());
-    }
+    setDesiredSize(window->windowContentGeometry().size());
 }
 
 QWaylandLayerSurface::~QWaylandLayerSurface()
@@ -99,7 +91,7 @@ void QWaylandLayerSurface::zwlr_layer_surface_v1_configure(uint32_t serial, uint
 
     if (!m_configured) {
         m_configured = true;
-        window()->resizeFromApplyConfigure(m_pendingSize);
+        applyConfigure();
         sendExpose();
     } else {
         // Later configures are resizes, so we have to queue them up for a time when we
@@ -121,13 +113,29 @@ void QWaylandLayerSurface::attachPopup(QtWaylandClient::QWaylandShellSurface *po
 
 void QWaylandLayerSurface::applyConfigure()
 {
+    m_configuring = true;
     window()->resizeFromApplyConfigure(m_pendingSize);
+    m_configuring = false;
+}
+
+void QWaylandLayerSurface::setDesiredSize(const QSize &size)
+{
+    const bool horizontallyConstrained = m_interface->anchors().testFlags({Window::AnchorLeft, Window::AnchorRight});
+    const bool verticallyConstrained = m_interface->anchors().testFlags({Window::AnchorTop, Window::AnchorBottom});
+
+    QSize effectiveSize = size;
+    if (horizontallyConstrained) {
+        effectiveSize.setWidth(0);
+    }
+    if (verticallyConstrained) {
+        effectiveSize.setHeight(0);
+    }
+    set_size(effectiveSize.width(), effectiveSize.height());
 }
 
 void QWaylandLayerSurface::setAnchor(uint anchor)
 {
     set_anchor(anchor);
-    setWindowGeometry(window()->windowContentGeometry());
 }
 
 void QWaylandLayerSurface::setExclusiveZone(int32_t zone)
@@ -160,22 +168,11 @@ void QWaylandLayerSurface::setLayer(uint32_t layer)
 
 void QWaylandLayerSurface::setWindowGeometry(const QRect &geometry)
 {
-    // if we are setting it to the last size we were configured at, we don't need to do anything
-    if (geometry.size() == m_pendingSize && !m_waitForSyncCallback) {
+    if (m_configuring) {
         return;
     }
 
-    const bool horizontallyConstrained = m_interface->anchors().testFlags({Window::AnchorLeft, Window::AnchorRight});
-    const bool verticallyConstrained = m_interface->anchors().testFlags({Window::AnchorTop, Window::AnchorBottom});
-
-    QSize size = geometry.size();
-    if (horizontallyConstrained) {
-        size.setWidth(0);
-    }
-    if (verticallyConstrained) {
-        size.setHeight(0);
-    }
-    set_size(size.width(), size.height());
+    setDesiredSize(geometry.size());
     requestWaylandSync();
 }
 
